@@ -1,6 +1,8 @@
 from typing import List, Optional, Union
 
-from terra_sdk.core import AccAddress, Coin, Coins, Numeric
+import attr
+
+from terra_sdk.core import AccAddress, Coins, Numeric
 from terra_sdk.core.auth import StdFee, StdSignMsg, StdTx, TxInfo
 from terra_sdk.core.broadcast import (
     AsyncTxBroadcastResult,
@@ -12,7 +14,13 @@ from terra_sdk.util.hash import hash_amino
 
 from ._base import BaseAsyncAPI, sync_bind
 
-__all__ = ["AsyncTxAPI", "TxAPI"]
+__all__ = ["AsyncTxAPI", "TxAPI", "BroadcastOptions"]
+
+
+@attr.s
+class BroadcastOptions:
+    sequences: Optional[List[int]] = attr.ib()
+    fee_granter: Optional[AccAddress] = attr.ib(default=None)
 
 
 class AsyncTxAPI(BaseAsyncAPI):
@@ -29,10 +37,11 @@ class AsyncTxAPI(BaseAsyncAPI):
 
     async def create(
         self,
-        source_address: AccAddress,
+        sender: AccAddress,
         msgs: List[Msg],
         fee: Optional[StdFee] = None,
         memo: str = "",
+        gas: Optional[int] = None,
         gas_prices: Optional[Coins.Input] = None,
         gas_adjustment: Optional[Numeric.Input] = None,
         fee_denoms: Optional[List[str]] = None,
@@ -43,7 +52,7 @@ class AsyncTxAPI(BaseAsyncAPI):
         chain ID, account number, sequence and fee estimation.
 
         Args:
-            source_address (AccAddress): transaction sender's account address
+            sender (AccAddress): transaction sender's account address
             msgs (List[Msg]): list of messages to include
             fee (Optional[StdFee], optional): fee to use (estimates if empty).
             memo (str, optional): memo to use. Defaults to "".
@@ -59,18 +68,14 @@ class AsyncTxAPI(BaseAsyncAPI):
 
         # create the fake fee
         if fee is None:
-            balance_denoms = fee_denoms or []
-            balance_one = [Coin(c, 1) for c in balance_denoms]
-            # estimate the fee
-            tx = StdTx(msgs, StdFee(0, balance_one), [], memo)
             fee = await BaseAsyncAPI._try_await(
-                self.estimate_fee(tx, gas_prices, gas_adjustment, fee_denoms)
+                self.estimate_fee(
+                    sender, msgs, memo, gas, gas_prices, gas_adjustment, fee_denoms
+                )
             )
 
         if account_number is None or sequence is None:
-            account = await BaseAsyncAPI._try_await(
-                self._c.auth.account_info(source_address)
-            )
+            account = await BaseAsyncAPI._try_await(self._c.auth.account_info(sender))
             if account_number is None:
                 account_number = account.account_number
             if sequence is None:
@@ -82,7 +87,10 @@ class AsyncTxAPI(BaseAsyncAPI):
 
     async def estimate_fee(
         self,
-        tx: Union[StdSignMsg, StdTx],
+        sender: AccAddress,
+        msgs: List[Msg],
+        memo: str = "",
+        gas: Optional[int] = None,
         gas_prices: Optional[Coins.Input] = None,
         gas_adjustment: Optional[Numeric.Input] = None,
         fee_denoms: Optional[List[str]] = None,
@@ -101,13 +109,6 @@ class AsyncTxAPI(BaseAsyncAPI):
         gas_prices = gas_prices or self._c.gas_prices
         gas_adjustment = gas_adjustment or self._c.gas_adjustment
 
-        if isinstance(tx, StdSignMsg):
-            tx_value = tx.to_stdtx().to_data()["value"]
-        else:
-            tx_value = tx.to_data()["value"]
-
-        tx_value["fee"]["gas"] = "0"
-
         gas_prices_coins = None
         if gas_prices:
             gas_prices_coins = Coins(gas_prices)
@@ -118,17 +119,23 @@ class AsyncTxAPI(BaseAsyncAPI):
                 )
 
         data = {
-            "tx": tx_value,
-            "gas_prices": gas_prices_coins and gas_prices_coins.to_data(),
-            "gas_adjustment": gas_adjustment and str(gas_adjustment),
+            "base_req": {
+                "chain_id": self._c.chain_id,
+                "from": sender,
+                "gas": (gas and str(gas)) or "auto",
+                "memo": memo,
+                "gas_prices": gas_prices_coins and gas_prices_coins.to_data(),
+                "gas_adjustment": gas_adjustment and str(gas_adjustment),
+            },
+            "msgs": [m.to_data() for m in msgs],
         }
 
         res = await self._c._post("/txs/estimate_fee", data)
-        fees = Coins.from_data(res["fees"])
+        fee_amount = Coins.from_data(res["fee"]["amount"])
 
-        return StdFee(int(res["gas"]), fees)
+        return StdFee(int(res["fee"]["gas"]), fee_amount)
 
-    async def encode(self, tx: StdTx) -> str:
+    async def encode(self, tx: StdTx, options: BroadcastOptions = None) -> str:
         """Fetches a transaction's amino encoding.
 
         Args:
@@ -137,7 +144,14 @@ class AsyncTxAPI(BaseAsyncAPI):
         Returns:
             str: base64 string containing amino-encoded tx
         """
-        res = await self._c._post("/txs/encode", tx.to_data())
+        data = tx.to_data()
+        if options is not None:
+            if options.sequences is not None and len(options.sequences) > 0:
+                data["sequences"] = [str(i) for i in options.sequences]
+            if options.fee_granter is not None and len(options.fee_granter) > 0:
+                data["fee_granter"] = options.fee_granter
+
+        res = await self._c._post("/txs/encode", data)
         return res["tx"]
 
     async def hash(self, tx: StdTx) -> str:
@@ -152,11 +166,20 @@ class AsyncTxAPI(BaseAsyncAPI):
         amino = await self.encode(tx)
         return hash_amino(amino)
 
-    async def _broadcast(self, tx: StdTx, mode: str) -> dict:
+    async def _broadcast(
+        self, tx: StdTx, mode: str, options: BroadcastOptions = None
+    ) -> dict:
         data = {"tx": tx.to_data()["value"], "mode": mode}
+        if options is not None:
+            if options.sequences is not None and len(options.sequences) > 0:
+                data["sequences"] = [str(i) for i in options.sequences]
+            if options.fee_granter is not None and len(options.fee_granter) > 0:
+                data["fee_granter"] = options.fee_granter
         return await self._c._post("/txs", data, raw=True)
 
-    async def broadcast_sync(self, tx: StdTx) -> SyncTxBroadcastResult:
+    async def broadcast_sync(
+        self, tx: StdTx, options: BroadcastOptions = None
+    ) -> SyncTxBroadcastResult:
         """Broadcasts a transaction using the ``sync`` broadcast mode.
 
         Args:
@@ -165,16 +188,17 @@ class AsyncTxAPI(BaseAsyncAPI):
         Returns:
             SyncTxBroadcastResult: result
         """
-        res = await self._broadcast(tx, "sync")
+        res = await self._broadcast(tx, "sync", options)
         return SyncTxBroadcastResult(
-            height=res["height"],
-            txhash=res["txhash"],
+            txhash=res.get("txhash"),
             raw_log=res.get("raw_log"),
             code=res.get("code"),
             codespace=res.get("codespace"),
         )
 
-    async def broadcast_async(self, tx: StdTx) -> AsyncTxBroadcastResult:
+    async def broadcast_async(
+        self, tx: StdTx, options: BroadcastOptions = None
+    ) -> AsyncTxBroadcastResult:
         """Broadcasts a transaction using the ``async`` broadcast mode.
 
         Args:
@@ -183,13 +207,14 @@ class AsyncTxAPI(BaseAsyncAPI):
         Returns:
             AsyncTxBroadcastResult: result
         """
-        res = await self._broadcast(tx, "async")
+        res = await self._broadcast(tx, "async", options)
         return AsyncTxBroadcastResult(
-            height=res["height"],
-            txhash=res["txhash"],
+            txhash=res.get("txhash"),
         )
 
-    async def broadcast(self, tx: StdTx) -> BlockTxBroadcastResult:
+    async def broadcast(
+        self, tx: StdTx, options: BroadcastOptions = None
+    ) -> BlockTxBroadcastResult:
         """Broadcasts a transaction using the ``block`` broadcast mode.
 
         Args:
@@ -198,10 +223,10 @@ class AsyncTxAPI(BaseAsyncAPI):
         Returns:
             BlockTxBroadcastResult: result
         """
-        res = await self._broadcast(tx, "block")
+        res = await self._broadcast(tx, "block", options)
         return BlockTxBroadcastResult(
-            height=res["height"],
-            txhash=res["txhash"],
+            height=res.get("height") or 0,
+            txhash=res.get("txhash"),
             raw_log=res.get("raw_log"),
             gas_wanted=res.get("gas_wanted") or 0,
             gas_used=res.get("gas_used") or 0,
@@ -233,7 +258,7 @@ class TxAPI(AsyncTxAPI):
     @sync_bind(AsyncTxAPI.create)
     def create(
         self,
-        source_address: AccAddress,
+        sender: AccAddress,
         msgs: List[Msg],
         fee: Optional[StdFee] = None,
         memo: str = "",
@@ -260,7 +285,7 @@ class TxAPI(AsyncTxAPI):
     estimate_fee.__doc__ = AsyncTxAPI.estimate_fee.__doc__
 
     @sync_bind(AsyncTxAPI.encode)
-    def encode(self, tx: StdTx) -> str:
+    def encode(self, tx: StdTx, options: BroadcastOptions = None) -> str:
         pass
 
     encode.__doc__ = AsyncTxAPI.encode.__doc__
@@ -272,19 +297,25 @@ class TxAPI(AsyncTxAPI):
     hash.__doc__ = AsyncTxAPI.hash.__doc__
 
     @sync_bind(AsyncTxAPI.broadcast_sync)
-    def broadcast_sync(self, tx: StdTx) -> SyncTxBroadcastResult:
+    def broadcast_sync(
+        self, tx: StdTx, options: BroadcastOptions = None
+    ) -> SyncTxBroadcastResult:
         pass
 
     broadcast_sync.__doc__ = AsyncTxAPI.broadcast_sync.__doc__
 
     @sync_bind(AsyncTxAPI.broadcast_async)
-    def broadcast_async(self, tx: StdTx) -> AsyncTxBroadcastResult:
+    def broadcast_async(
+        self, tx: StdTx, options: BroadcastOptions = None
+    ) -> AsyncTxBroadcastResult:
         pass
 
     broadcast_async.__doc__ = AsyncTxAPI.broadcast_async.__doc__
 
     @sync_bind(AsyncTxAPI.broadcast)
-    def broadcast(self, tx: StdTx) -> BlockTxBroadcastResult:
+    def broadcast(
+        self, tx: StdTx, options: BroadcastOptions = None
+    ) -> BlockTxBroadcastResult:
         pass
 
     broadcast.__doc__ = AsyncTxAPI.broadcast.__doc__
