@@ -1,8 +1,8 @@
 from __future__ import annotations
 
 from asyncio import AbstractEventLoop, get_event_loop
+from json import JSONDecodeError
 from typing import Optional, Union
-from urllib.parse import urljoin
 
 import nest_asyncio
 from aiohttp import ClientSession
@@ -11,6 +11,7 @@ from terra_sdk.core import Coins, Dec, Numeric
 from terra_sdk.exceptions import LCDResponseError
 from terra_sdk.key.key import Key
 from terra_sdk.util.json import dict_to_data
+from terra_sdk.util.url import urljoin
 
 from .api.auth import AsyncAuthAPI, AuthAPI
 from .api.bank import AsyncBankAPI, BankAPI
@@ -86,13 +87,13 @@ class AsyncLCDClient:
         async with self.session.get(
             urljoin(self.url, endpoint), params=params
         ) as response:
-            result = await response.json(content_type=None)
-            if not str(response.status).startswith("2"):
+            try:
+                result = await response.json(content_type=None)
+            except JSONDecodeError:
+                raise LCDResponseError(message=str(response.reason), response=response)
+            if not 200 <= response.status < 299:
                 raise LCDResponseError(message=result.get("error"), response=response)
-        try:
-            self.last_request_height = result["height"]
-        except KeyError:
-            self.last_request_height = None
+        self.last_request_height = result.get("height")
         return result if raw else result["result"]
 
     async def _post(
@@ -101,13 +102,13 @@ class AsyncLCDClient:
         async with self.session.post(
             urljoin(self.url, endpoint), json=data and dict_to_data(data)
         ) as response:
-            result = await response.json(content_type=None)
-            if not str(response.status).startswith("2"):
+            try:
+                result = await response.json(content_type=None)
+            except JSONDecodeError:
+                raise LCDResponseError(message=str(response.reason), response=response)
+            if not 200 <= response.status < 299:
                 raise LCDResponseError(message=result.get("error"), response=response)
-        try:
-            self.last_request_height = result["height"]
-        except KeyError:
-            self.last_request_height = None
+        self.last_request_height = result.get("height")
         return result if raw else result["result"]
 
     async def __aenter__(self):
