@@ -1,3 +1,5 @@
+"""Slashing module messages types."""
+
 from __future__ import annotations
 
 import attr
@@ -10,9 +12,15 @@ __all__ = ["MsgUnjail"]
 
 @attr.s
 class MsgUnjail(Msg):
+    """Attempt to unjail a jailed validator (must be submitted by same validator).
+
+    Args:
+        address: validator address to unjail"""
 
     type = "slashing/MsgUnjail"
+    """"""
     action = "unjail"
+    """"""
 
     address: ValAddress = attr.ib()
 
